@@ -58,11 +58,13 @@ def batched_index_select(values, indices, dim = 1):
 # https://github.com/lucidrains/se3-transformer-pytorch/blob/main/se3_transformer_pytorch/se3_transformer_pytorch.py#L95
 
 class CoorsNorm(nn.Module):
-    def __init__(self, eps = 1e-8):
+    def __init__(self, eps = 1e-8, scale_init = 1., bias_init = 0.):
         super().__init__()
         self.eps = eps
-        self.scale = nn.Parameter(torch.ones(1))
-        self.bias = nn.Parameter(torch.zeros(1))
+        scale = torch.zeros(1).fill_(scale_init)
+        bias = torch.zeros(1).fill_(bias_init)
+        self.scale = nn.Parameter(scale)
+        self.bias = nn.Parameter(bias)
 
     def forward(self, coors):
         norm = coors.norm(dim = -1, keepdim = True)
@@ -153,13 +155,15 @@ class GlobalLinearAttention(nn.Module):
         dim_head = 64
     ):
         super().__init__()
+        self.norm = nn.LayerNorm(dim)
         self.attn1 = Attention(dim, heads, dim_head)
         self.attn2 = Attention(dim, heads, dim_head)
 
     def forward(self, x, queries, mask = None):
+        queries = self.norm(queries)
         induced = self.attn1(queries, x, mask = mask)
         out     = self.attn2(x, induced)
-        return out, 0
+        return out, induced
 
 class EquivariantAttention(nn.Module):
     def __init__(
@@ -176,7 +180,9 @@ class EquivariantAttention(nn.Module):
         init_eps = 1e-3,
         rel_pos_emb = None,
         edge_mlp_mult = 2,
-        norm_rel_coors = True
+        norm_rel_coors = True,
+        norm_coors_scale_init = 1.,
+        norm_coors_bias_init = 0.
     ):
         super().__init__()
         self.scale = dim_head ** -0.5
@@ -219,7 +225,7 @@ class EquivariantAttention(nn.Module):
             nn.Tanh()
         )
 
-        self.norm_rel_coors = CoorsNorm() if norm_rel_coors else nn.Identity()
+        self.norm_rel_coors = CoorsNorm(scale_init = norm_coors_scale_init, bias_init = norm_coors_bias_init) if norm_rel_coors else nn.Identity()
         self.coors_combine = nn.Parameter(torch.randn(heads))
 
         self.rotary_emb = SinusoidalEmbeddings(dim_head // 2)
@@ -419,6 +425,8 @@ class EnTransformer(nn.Module):
         valid_neighbor_radius = float('inf'),
         init_eps = 1e-3,
         norm_rel_coors = True,
+        norm_coors_scale_init = 1.,
+        norm_coors_bias_init = 0.,
         global_linear_attn_every = 0,
         num_global_tokens = 8
     ):
@@ -450,11 +458,10 @@ class EnTransformer(nn.Module):
 
             self.layers.append(nn.ModuleList([
                 nn.ModuleList([
-                    nn.LayerNorm(dim),
-                    PreNorm(dim, GlobalLinearAttention(dim = dim, heads = heads, dim_head = dim_head)),
+                    Residual(PreNorm(dim, GlobalLinearAttention(dim = dim, heads = heads, dim_head = dim_head))),
                     Residual(PreNorm(dim, FeedForward(dim = dim))),
                 ])  if add_global else None,
-                Residual(PreNorm(dim, EquivariantAttention(dim = dim, dim_head = dim_head, heads = heads, coors_hidden_dim = coors_hidden_dim, edge_dim = (edge_dim + adj_dim),  neighbors = neighbors, only_sparse_neighbors = only_sparse_neighbors, valid_neighbor_radius = valid_neighbor_radius, init_eps = init_eps, rel_pos_emb = rel_pos_emb, norm_rel_coors = norm_rel_coors))),
+                Residual(PreNorm(dim, EquivariantAttention(dim = dim, dim_head = dim_head, heads = heads, coors_hidden_dim = coors_hidden_dim, edge_dim = (edge_dim + adj_dim),  neighbors = neighbors, only_sparse_neighbors = only_sparse_neighbors, valid_neighbor_radius = valid_neighbor_radius, init_eps = init_eps, rel_pos_emb = rel_pos_emb, norm_rel_coors = norm_rel_coors, norm_coors_scale_init = norm_coors_scale_init, norm_coors_bias_init = norm_coors_bias_init))),
                 Residual(PreNorm(dim, FeedForward(dim = dim)))
             ]))
 
@@ -506,13 +513,9 @@ class EnTransformer(nn.Module):
 
         for global_fns, attn, ff in self.layers:
             if exists(global_fns):
-                global_norm, global_attn, global_ff = global_fns
-
-                feats_out, global_out = global_attn(feats, global_norm(global_tokens), mask = mask)
-
-                feats = feats + feats_out
-                global_tokens = global_tokens + global_out
+                global_attn, global_ff = global_fns
 
+                feats, global_tokens = global_attn(feats, global_tokens, mask = mask)
                 feats, coors = global_ff(feats, coors)
 
             feats, coors = attn(feats, coors, edges = edges, mask = mask, adj_mat = adj_mat)
